import { Application, Router } from 'https://deno.land/x/oak/mod.ts';
import { Client } from "https://deno.land/x/postgres@v0.17.0/mod.ts";

const app = new Application();
const router = new Router();

//---------dotenv info-------------
import { load } from "https://deno.land/std/dotenv/mod.ts";

const env = await load();
const hostname = env["HOSTNAME"];
const port     = env["PORT"];
const user     = env["USER"];
const password = env["PASSWORD"];
const database = env["DATABASE"];
const app_port = env["APP_PORT"];


// Configure database connection
const client = new Client({
  hostname:hostname 
  ,port:    port     
  ,user:    user     
  ,password:password 
  ,database:database 
  ,applicationName: "pricing guidance"
});

await client.connect();

// Load SQL from file
const query = await Deno.readTextFile("sql/get.pg.sql");

function apply_guidance(doc: any) {
  let mostRelevantPrice = null;
  let mostRelevantKey = null;
  let highestRelevanceLevel = -1; // Assume -1 is less than any relevance level

  // Function to update price and assign relevance indicator
  function updatePriceAndAssignRelevance(items, channelFirstChar, v1ds, v0ds, histKey) {
    for (let item of items) {
      // Update the last_price with the most recent price
      const years = Object.keys(item.season);
      if (years.length > 0) {
        const recentYear = Math.max(...years.map(Number));
        const lastPrice = item.season[recentYear].price_usd;
        item.last_price = lastPrice;
      } else {
        item.last_price = null; // or some default value as appropriate
      }

      // Initialize relevance as numeric value
      let relevance = 0; // Assume 0 is 'not relevant'

      // Check if the first character of the item's channel matches the first character of the document's channel
      if (item.chan.charAt(0) === channelFirstChar) {
        relevance = 1; // 'relevant'

        // Further refine relevance based on v1ds and v0ds
        if (item.v1ds === v1ds) {
          relevance = 2; // 'most relevant' because v1ds matches
        } else if (item.v0ds === v0ds) {
          relevance = relevance === 2 ? 2 : 1; // Keep relevance as is if v1ds was matched, otherwise it's just 'relevant'
        }
      }

      // Assign the calculated relevance to the item
      item.relevance = relevance;

      // Update the most relevant price if this item's relevance is higher
      if (relevance > highestRelevanceLevel) {
        highestRelevanceLevel = relevance;
        mostRelevantPrice = item.last_price;
        mostRelevantKey = histKey;
      }
    }
  }

  // Iterate over each key in the "hist" object
  for (let key of Object.keys(doc.hist)) {
    // Update price and relevance for each item in the current key
    updatePriceAndAssignRelevance(doc.hist[key], doc.chan[0], doc.v1ds, doc.v0ds, key);
  }

  // Assign the most relevant price and key to the top level of the document
  if (mostRelevantPrice !== null) {
    doc.mostRelevantPriceInfo = {
      price: mostRelevantPrice,
      histKey: mostRelevantKey
    };
  }

  return doc;
}


// Function to update each item with the most recent price
function updateWithMostRecentPrice(items) {
  for (let item of items) {
    // Find the most recent price
    //const lastPrice = findMostRecentPrice(item.season);
    const years = Object.keys(item.season);
    const recentYear = Math.max(...years);
    const lastPrice = item.season[recentYear].price_usd;
    // Append the last_price to the item
    item.last_price = lastPrice;
  }
}

// Define a route to retrieve values from the database using parameters
router.get('/code_price/:billcode/:shipcode/:partcode/:qty', async (ctx) => {

    const partcode = ctx.params.partcode;
    const billcode = ctx.params.billcode;
    const shipcode = ctx.params.shipcode;
    const qty = ctx.params.qty;

    //console.log(partcode)
    //console.log(customer)
    const result = await client.queryObject({args: [billcode, shipcode, partcode, qty], text: query} );
    const procd = apply_guidance(result.rows[0]["doc"])
    ctx.response.body = procd
});

app.use(router.routes());
app.use(router.allowedMethods());

// Start the server
console.log('Server is running on http://usmidsap02:8090');
await app.listen({ port: 8090 });

